﻿-- ********************************************************************************
-- Broker Statistics
-- A Data Broker display for combat statistics.
-- By: Shenton
--
-- IconsFrame.lua
-- Icons selection frame.
-- ********************************************************************************

local A = _G["DataBrokerStatisticsGlobal"];

-- ********************************************************************************
-- Variables & constants
-- ********************************************************************************

local strupper, smatch, gsub = strupper, string.match, gsub;
local ceil = ceil;

local iconsList = {};
local iconsListSearch = {};

-- ********************************************************************************
-- Functions
-- ********************************************************************************

--- Add spellbook icons to the icons list table
function A:GetSpellsIcons()
    local offset, numSpells, spellBookType, spellBookId, spellTexture, numSlots, isKnown, spellId, _;

    for i=1,GetNumSpellTabs() do
        _, _, offset, numSpells = GetSpellTabInfo(i);

        for ii=offset+1,offset+numSpells do
            spellBookType, spellBookId = GetSpellBookItemInfo(ii, "player");

            if ( spellBookType ~= "FUTURESPELL" ) then
                spellTexture = strupper(GetSpellBookItemTexture(ii, "player"));

                if ( not smatch(spellTexture, "INTERFACE\\BUTTONS\\") ) then
                    iconsList[#iconsList+1] = gsub(spellTexture, "INTERFACE\\ICONS\\", "");
                end
            end

            if (spellBookType == "FLYOUT") then
                _, _, numSlots, isKnown = GetFlyoutInfo(spellBookId);

                if (isKnown and numSlots > 0) then
                    for iii = 1, numSlots do 
                        local spellId, isKnown = GetFlyoutSlotInfo(spellBookId, iii);

                        if (isKnown) then
                            iconsList[#iconsList+1] = gsub(strupper(GetSpellTexture(spellId)), "INTERFACE\\ICONS\\", "");
                        end
                    end
                end
            end
        end
    end
end

--- Reset and add icons filename to the icons list table
function A:GetIconsList()
    iconsList = {};
    iconsList[1] = "INV_MISC_QUESTIONMARK";
    --A:GetSpellsIcons();
    GetMacroIcons(iconsList);
    GetMacroItemIcons(iconsList);
end

--- Icon search method
function A:IconSearch(text)
    iconsListSearch = {};

    if ( not text or text == "" ) then return; end

    local gotOne;
    text = string.lower(text);

    for k,v in pairs(iconsList) do
        if ( string.find(string.lower(v), text) ) then
            iconsListSearch[#iconsListSearch+1] = v;
            gotOne = 1;
        end
    end

    if ( not gotOne ) then
        iconsListSearch[1] = "INV_MISC_QUESTIONMARK";
    end
end

--- Init the icon selection frame and update when scrolling (Blizzard guild bank name/icon selection frame)
function A:PopupScrollUpdate()
    local icons, icon, button, index, texture;
    local popupOffset = FauxScrollFrame_GetOffset(BrokerStatisticsPopupFrameScrollFrame);

    if ( #iconsListSearch > 0 ) then
        icons = #iconsListSearch;
    else
        icons = #iconsList;
    end

    for i=1,64 do
        icon = _G["BrokerStatisticsPopupFrameButton"..i.."Icon"];
        button = _G["BrokerStatisticsPopupFrameButton"..i];
        index = (popupOffset * 4) + i;

        if ( index ) then
            if ( #iconsListSearch > 0 ) then
                texture = iconsListSearch[index];
            else
                texture = iconsList[index];
            end
        else
            texture = nil;
        end

        if ( index <= icons and texture ) then
            icon:SetTexture("INTERFACE\\ICONS\\"..texture);
            button.textureName = texture;
            button:Show();
        else
            icon:SetTexture("");
            button.textureName = nil;
            button:Hide();
        end

        if ( BrokerStatisticsPopupFrame.selectedIcon ) then
            if ( index == BrokerStatisticsPopupFrame.selectedIcon ) then
                button:SetChecked(1);
            else
                button:SetChecked(nil);
            end
        elseif ( BrokerStatisticsPopupFrame.currentTexture == texture ) then
            button:SetChecked(1);
            BrokerStatisticsPopupFrame.selectedIcon = index;
        else
            button:SetChecked(nil);
        end
    end

    if ( not BrokerStatisticsPopupFrame.selectedIcon ) then
        for i=1, icons do
            if ( #iconsListSearch > 0 ) then
                texture = iconsListSearch[index];
            else
                texture = iconsList[index];
            end

            if ( BrokerStatisticsPopupFrame.currentTexture == texture ) then
                BrokerStatisticsPopupFrame.selectedIcon = i;
                break;
            end
        end
    end

    FauxScrollFrame_Update(BrokerStatisticsPopupFrameScrollFrame, ceil(icons / 4) , 4, 36);
end

--- When an icon is clicked
function A:PopupScrollButtonClick(self)
    local offset = FauxScrollFrame_GetOffset(BrokerStatisticsPopupFrameScrollFrame);
    local index = (offset * 4) + self:GetID();
    BrokerStatisticsPopupFrame.selectedIcon = index;
    A:PopupScrollUpdate();
end

-- Frame
function A:IconsFrame()
    local f = CreateFrame("Frame", "BrokerStatisticsPopupFrame", UIParent);
    f:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        tile = true, tileSize = 32, edgeSize = 32,
        insets = { left = 8, right = 8, top = 8, bottom = 8 }});
    f:SetClampedToScreen(1);
    f:EnableMouse(1);
    f:SetFrameStrata("FULLSCREEN_DIALOG");
    f:Hide();
    f:SetSize(416, 500);

    -- Textures
    -- f.topLeft = f:CreateTexture();
    -- f.topLeft:SetTexture("Interface\\MacroFrame\\MacroPopup-TopLeft");
    -- f.topLeft:SetSize(350, 512);
    -- f.topLeft:SetPoint("TOPLEFT", f);
    -- f.topLeft:SetDrawLayer("BACKGROUND");

    -- f.topleft = f:CreateTexture();
    -- f.topleft:SetTexture("Interface\\MacroFrame\\MacroPopup-TopRight");
    -- f.topleft:SetSize(100, 512);
    -- f.topleft:SetPoint("TOPRIGHT", f);
    -- f.topleft:SetDrawLayer("BACKGROUND");

    -- f.topleft = f:CreateTexture();
    -- f.topleft:SetTexture("Interface\\MacroFrame\\MacroPopup-BotLeft");
    -- f.topleft:SetSize(350, 128);
    -- f.topleft:SetPoint("BOTTOMLEFT", f);
    -- f.topleft:SetDrawLayer("BACKGROUND");

    -- f.topleft = f:CreateTexture();
    -- f.topleft:SetTexture("Interface\\MacroFrame\\MacroPopup-BotRight");
    -- f.topleft:SetSize(100, 128);
    -- f.topleft:SetPoint("BOTTOMRIGHT", f);
    -- f.topleft:SetDrawLayer("BACKGROUND");

    -- Text
    f.text = f:CreateFontString("BrokerStatisticsPopupFrameText");
    f.text:SetFontObject(SystemFont_Med1);
    f.text:SetText(A.L["Search"]);
    f.text:SetPoint("TOPLEFT", f, 16, -16);

    -- Search box
    f.search = CreateFrame("EditBox", "BrokerStatisticsPopupFrameSearchBox", f, "SearchBoxTemplate");
    f.search:SetSize(256, 48);
    f.search:SetPoint("TOPLEFT", f, 20, -20);
    f.search:SetScript("OnTextChanged", function(self)
        local text = self:GetText();

        A:IconSearch(text);
        A:PopupScrollUpdate();
    end);

    -- Scroll
    f.scroll = CreateFrame("ScrollFrame", "BrokerStatisticsPopupFrameScrollFrame", f, "ListScrollFrameTemplate");
    f.scroll:SetSize(370, 350);
    f.scroll:SetPoint("RIGHT", f, "RIGHT", -34, -34);
    f.scroll:SetScript("OnVerticalScroll", function(self, offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, 36, A.PopupScrollUpdate);
    end);

    -- Icons buttons
    f.button1 = CreateFrame("CheckButton", "BrokerStatisticsPopupFrameButton1", f, "PopupButtonTemplate");
    f.button1:SetID(1);
    f.button1:SetPoint("TOPLEFT", f, "TOPLEFT", 16, -110);
    f.button1:SetScript("OnClick", function(self) A:PopupScrollButtonClick(self); end);
    f.button1:HookScript("OnEnter", function(self)
        if ( self:GetRight() >= ( GetScreenWidth() / 2 ) ) then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT");
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
        end

        GameTooltip:AddLine(self.textureName);
        GameTooltip:Show();
    end);
    f.button1:HookScript("OnLeave", function(self)
        GameTooltip:Hide();
    end);

    local button, row = 2, 1;
    while ( button <= 64 ) do
        if ( row == 8 ) then
            f["button"..button] = CreateFrame("CheckButton", "BrokerStatisticsPopupFrameButton"..button, _G["BrokerStatisticsPopupFrameButton"..button - 4], "PopupButtonTemplate");
            f["button"..button]:SetID(button);
            f["button"..button]:SetPoint("TOPLEFT", _G["BrokerStatisticsPopupFrameButton"..button - 8], "BOTTOMLEFT", 0, -8);
            f["button"..button]:SetScript("OnClick", function(self) A:PopupScrollButtonClick(self); end);
            f["button"..button]:HookScript("OnEnter", function(self)
                if ( self:GetRight() >= ( GetScreenWidth() / 2 ) ) then
                    GameTooltip:SetOwner(self, "ANCHOR_LEFT");
                else
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
                end

                GameTooltip:AddLine(self.textureName);
                GameTooltip:Show();
            end);
            f["button"..button]:HookScript("OnLeave", function(self)
                GameTooltip:Hide();
            end);
            row = 1;
        else
            f["button"..button] = CreateFrame("CheckButton", "BrokerStatisticsPopupFrameButton"..button, _G["BrokerStatisticsPopupFrameButton"..button - 1], "PopupButtonTemplate");
            f["button"..button]:SetID(button);
            f["button"..button]:SetPoint("LEFT", _G["BrokerStatisticsPopupFrameButton"..button - 1], "RIGHT", 10, 0);
            f["button"..button]:SetScript("OnClick", function(self) A:PopupScrollButtonClick(self); end);
            f["button"..button]:HookScript("OnEnter", function(self)
                if ( self:GetRight() >= ( GetScreenWidth() / 2 ) ) then
                    GameTooltip:SetOwner(self, "ANCHOR_LEFT");
                else
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
                end

                GameTooltip:AddLine(self.textureName);
                GameTooltip:Show();
            end);
            f["button"..button]:HookScript("OnLeave", function(self)
                GameTooltip:Hide();
            end);
            row = row + 1;
        end

        button = button + 1;
    end

    -- Cancel button
    f.cancelButton = CreateFrame("Button", "BrokerStatisticsPopupFrameButtonCancel", f, "UIPanelButtonTemplate");
    f.cancelButton:SetSize(69, 22);
    f.cancelButton:SetText(A.L["Cancel"]);
    f.cancelButton:SetScript("OnClick", function()
        BrokerStatisticsPopupFrame:Hide();
        BrokerStatisticsPopupFrame.ldbObject = nil;
        BrokerStatisticsPopupFrame.selectedIcon = nil;
        --BrokerStatisticsPopupFrame.iconList = nil;
    end);
    f.cancelButton:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -16, 16);

    -- Accept button
    f.acceptButton = CreateFrame("Button", "BrokerStatisticsPopupFrameButtonAccept", BrokerStatisticsPopupFrameButtonCancel, "UIPanelButtonTemplate");
    f.acceptButton:SetSize(69, 22);
    f.acceptButton:SetText(A.L["Accept"]);
    f.acceptButton:SetScript("OnClick", function()
        if ( not BrokerStatisticsPopupFrame.selectedIcon ) then
            A:Message(A.L["No icon selected."], 1, 1);
        else
            local texture;

            BrokerStatisticsPopupFrame:Hide();

            texture = "INTERFACE\\ICONS\\"..iconsList[BrokerStatisticsPopupFrame.selectedIcon];
            if ( #iconsListSearch > 0 ) then
                texture = "INTERFACE\\ICONS\\"..iconsListSearch[BrokerStatisticsPopupFrame.selectedIcon];
            else
                texture = "INTERFACE\\ICONS\\"..iconsList[BrokerStatisticsPopupFrame.selectedIcon];
            end

            BrokerStatisticsPopupFrame.dbEdit.icon = texture;
            if ( BrokerStatisticsPopupFrame.ldbObject ) then
                BrokerStatisticsPopupFrame.ldbObject.icon = texture;
            end

            BrokerStatisticsPopupFrame.dbEdit = nil;
            BrokerStatisticsPopupFrame.ldbObject = nil;
            BrokerStatisticsPopupFrame.selectedIcon = nil;
            --BrokerStatisticsPopupFrame.iconList = nil;

            A:RefreshConfig();
        end
    end);
    f.acceptButton:SetPoint("RIGHT", BrokerStatisticsPopupFrameButtonCancel, "LEFT", -2, 0);

    -- Default button
    f.acceptButton = CreateFrame("Button", "BrokerStatisticsPopupFrameButtonDefault", BrokerStatisticsPopupFrameButtonAccept, "UIPanelButtonTemplate");
    f.acceptButton:SetSize(69, 22);
    f.acceptButton:SetText(A.L["Default"]);
    f.acceptButton:SetScript("OnClick", function()
        BrokerStatisticsPopupFrame:Hide();

        BrokerStatisticsPopupFrame.dbEdit.icon = A.defaultLdbIcon;
        if ( BrokerStatisticsPopupFrame.ldbObject ) then
            BrokerStatisticsPopupFrame.ldbObject.icon = A.defaultLdbIcon;
        end

        BrokerStatisticsPopupFrame.dbEdit = nil;
        BrokerStatisticsPopupFrame.ldbObject = nil;
        BrokerStatisticsPopupFrame.selectedIcon = nil;
        --BrokerStatisticsPopupFrame.iconList = nil;

        A:RefreshConfig();
    end);
    f.acceptButton:SetPoint("RIGHT", BrokerStatisticsPopupFrameButtonAccept, "LEFT", -2, 0);

    f:SetScript("OnShow", function()
        f.search:SetText("");
        A:GetIconsList();
    end);
end
